// IORDACHE CATALIN 333CA
// PROIECT - ELECTRIC PIANO - PROIECTAREA CU MICROPROCESOARE 2014
#include <avr/io.h>
#include <avr/interrupt.h>

#define F_CPU 16000000
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>

#define STATE_NORMAL 0
#define STATE_RECORD 1
#define STATE_PLAY   2

#define PAUSE_NOTE   0
#define SONG_LENGTH 100

struct Sound {
  uint8_t note;
  uint32_t count;
};

static uint8_t notes[8] = { 119, 106, 94, 89, 79, 70, 63, 59 };
static unsigned int state = 0, rec_pos = 0, rec_pressed = 0, play_pressed = 0,
    play_pos = 0;
static unsigned int blue_led_state = 0, green_led_state = 0;
static struct Sound REC[SONG_LENGTH];
static struct Sound PLAY[SONG_LENGTH];

static void init() {

  DDRC &= ~(1 << PC1); // pinul PC1 va fi PLAY
  PORTC |= (1 << PC1); // activam rezistenta de pull-up
  DDRC &= ~(1 << PC3); // pinul PC3 va fi REC
  PORTC |= (1 << PC3); // activam rezistenta de pull-up

  DDRC |= (1 << PC2); // pinul corespunzator led-ului PLAY (verde) este output
  DDRC |= (1 << PC4); // pinul corespunzator led-ului REC (albastru) este output

  /* Setam pinii de input pentru claviatura - PORTA */
  DDRA = 0x00;
  PORTA = 0xFF;

  /* Setare output pentru difuzor. Difuzorul este conectat la pinul PD5 */
  DDRD |= (1 << PD5);
  TCCR1A |= (1 << WGM10) | (1 << COM1A0);
  TCCR1B |= (1 << WGM13) | (1 << CS12);

  /* Modul CTC */
  TCCR2A = (1 << WGM21);
  TCCR2B |= (1 << CS22) | (1 << CS21) | (1 << CS20);

  /* La fiecare 10ms setam OCR */
  OCR2A = 156;

  /* Intreruperi */
  TIMSK1 |= (1 << OCIE1A);
}

ISR( TIMER1_COMPA_vect) {

  uint8_t note = 0;
  uint8_t is_playing = 0;

  /* Cand se apasa butonul de RECORD */
  if (!(PINC & (1 << PC3))) {

    if (rec_pressed == 0) {

      /* Ambele led-uri aprinse nu reprezinta nicio stare */
      if (green_led_state == 1) {

        green_led_state = play_pressed = 0;
        PORTC &= ~(1 << PC2); // GREEN LED OFF
        state = STATE_NORMAL;
      }

      /* Daca niciun LED nu este aprins - STATE_NORMAL, se intra in starea RECORD.
         LED-ul albstru va ramane aprins.*/
      if (state == STATE_NORMAL && blue_led_state == 0) {

        PORTC |= (1 << PC4); // BLUE LED ON
        state = STATE_RECORD;
        blue_led_state = 1;

        rec_pos = 0;
      }
      /* Daca suntem in starea RECORD si vrem sa iesim, mai apasam odata pe butonul de REC.
         LED-ul albastru se va stinge.*/
      else if (state == STATE_RECORD) {

        PORTC &= ~(1 << PC4); // BLUE LED OFF
        state = STATE_NORMAL;
        blue_led_state = 0;
      }
    }

    rec_pressed = 1;
  } else {

    rec_pressed = 0;
  }

  /* Cand se apasa butonul de PLAY*/
  if (!(PINC & (1 << PC1))) {

    if (play_pressed == 0) {

      /* Ambele led-uri aprinse nu reprezinta nicio stare*/
      if (blue_led_state == 1) {

        blue_led_state = rec_pressed = 0;
        PORTC &= ~(1 << PC4); // BLUE LED OFF
        state = STATE_NORMAL;
      }

      /* Daca niciun LED nu este aprins - STATE_NORMAL, se intra in starea PLAY.
         LED-ul verde va ramane aprins.*/
      if (state == STATE_NORMAL) {

        int i;

        PORTC |= (1 << PC2); // GREEN LED ON
        state = STATE_PLAY;
        green_led_state = 1;

        play_pos = 0;

        /* Copiem melodia inregistrata in vectorul PLAY*/
        for (i = 0; i < SONG_LENGTH; i++) {
          PLAY[i].note = REC[i].note;
          PLAY[i].count = REC[i].count;
        }

      }
      /* Daca suntem in starea PLAY si vrem sa iesim, mai apasam odata pe butonul de PLAY.
         LED-ul VERDE se va stinge.*/
      else if (state == STATE_PLAY
          || (state = STATE_NORMAL && green_led_state == 1)) {

        PORTC &= ~(1 << PC2); // GREEN LED OFF
        state = STATE_NORMAL;
        green_led_state = 0;
      }
    }

    play_pressed = 1;
  } else {

    play_pressed = 0;
  }

  /* Daca un buton de pe claviatura este apasat (PORTA)*/
  for (note = 0; note < 8; note++) {
    if (!(PINA & (1 << note))) {

      /*set note*/
      OCR1A = notes[note];
      /* Setam PWM */
      TCCR1A |= (1 << COM1A0);
      is_playing = 1;
      break;
    }
  }

  /* Daca nu s-a apasat niciun buton de pe claviatura la acest moment de timp*/
  if (is_playing == 0) {
    /* Unset PWM*/
    TCCR1A &= ~(1 << COM1A0);
    note = PAUSE_NOTE;
  }

  /* STATE_RECORD*/
  if (state == STATE_RECORD && rec_pos < SONG_LENGTH - 1) {

    if (note == REC[rec_pos].note) {

      REC[rec_pos].count++;
    } else {

      REC[++rec_pos].note = note;
      REC[rec_pos].count = 1;
    }
  }

  /* Cand numai este loc in memorie se intra in starea de PLAY*/
  if (state == STATE_RECORD && rec_pos >= SONG_LENGTH - 1) {

    state = STATE_PLAY;
  }

  /* Cand ne aflam in stare de PLAY*/
  if (state == STATE_PLAY && play_pos <= rec_pos) {

    /* Daca nota se repeta, o mai cantam odata*/
    if (PLAY[play_pos].count == 0) {
      play_pos++;
    }

    if (play_pos > rec_pos) {
      /* Unset PWM*/
      TCCR1A &= ~(1 << COM1A0);
      state = STATE_NORMAL;
      return;
    }

    if (PLAY[play_pos].note == PAUSE_NOTE) {
      /* Unset PWM*/
      TCCR1A &= ~(1 << COM1A0);
    } else {

      /* set note*/
      OCR1A = notes[PLAY[play_pos].note];
      /* Setam PWM */
      TCCR1A |= (1 << COM1A0);
    }

    PLAY[play_pos].count--;
  }

}

int main() {

  sei();
  init();

  /*set note*/
  OCR1A= notes[0];
  /* Setam PWM */
  TCCR1A |= (1 << COM1A0);

  while (1) {
    ;
  }

  return 0;
}